# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Guillaume Emont <guillaume@fluendo.com>

from twisted.internet import defer, task


def sequential_iteration(iterable, callable, *args, **kw):
    """
    DOCME
    """
    # FIXME: find a way to return the results while handling StopIteration
    def iterate():
        for item in iterable:
            dfr = defer.maybeDeferred(lambda:item)
            dfr.addCallback(callable, *args, **kw)
            yield dfr

    def iteration_end_errback(failure):
        failure.trap(StopIteration)

    dfr = task.coiterate(iterate())
    dfr.addErrback(iteration_end_errback)

    return dfr

def stop_iteration(result):
    raise StopIteration()

def parallel_iteration(iterable, callable, *args, **kw):
    """
    DOCME
    """
    dfr_list = []
    for item in iterable:
        dfr = defer.maybeDeferred(lambda:item)
        dfr.addCallback(callable, *args, **kw)
        dfr_list.append(dfr)

    dfr = defer.DeferredList(dfr_list, fireOnOneErrback=True,
                              consumeErrors=True)
    
    def errback(failure):
        # we convert the FirstError and lose the index information, for
        # consistency with sequential_iteration().
        failure.trap(defer.FirstError)
        return failure.value.subFailure

    dfr.addErrback(errback)
    return dfr

