import os
import urlparse, urllib

from crawler import Crawler

class FileSystemCrawler(Crawler):
    def crawl(self, uri):
        top_dir = self._uri_to_path(uri)
        if os.path.isdir(top_dir):
            for file_path in self._path_crawl(top_dir):
                yield self._path_to_uri(file_path)
        else:
            yield uri

    def _uri_to_path(self, uri):
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(uri)
        return urllib.unquote(path)

    def _path_to_uri(self, path):
        quoted_path = urllib.quote(path)
        # FIXME: we might need to add a / on non-unix platforms
        return "file://" + quoted_path

    def _can_read(self, path):
        return os.access(path, os.R_OK)

    def _path_crawl(self, dir):
        directory_content = os.listdir(dir)
        for object in directory_content:
            object_path = os.path.join(dir, object)
            if not self._can_read(object_path):
                continue
            if os.path.isdir(object_path):
                for file_path in self._path_crawl(object_path):
                    yield file_path
            else:
                yield object_path



if __name__ == '__main__':
    import sys
    uri = sys.argv[1]
    crawler = FileSystemCrawler()
    for uri in crawler.crawl(uri):
        print uri
