# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

import RDF

from core.media_scanner.fs_crawler import FileSystemCrawler
from plugins.typefinder.typefinder import MetaTypeFinder, FileTypeFinder
from core.media_scanner.identificator import VideoIdentificator
from core.media_scanner.rdf_model import RDFModel

from plugins.dbpedia.metadata_source import DBPediaDataSource
from plugins.themoviedb.metadata_source import TheMovieDBDataSource
from plugins.thetvdb.metadata_source import TheTVDBDataSource

from twisted.internet import reactor

rdfs = RDF.NS('http://www.w3.org/2000/01/rdf-schema#')

from twisted.internet import defer

from core.media_scanner.statistics import Statistics

class MediaScanner(object):
    """
    The Media Scanner is the objects that handles the logic behind the scanning
    of local files and media sources.
    """

    def __init__(self, debug_level):
        # super(Whatever, self).__init__()
        self.debug_level = debug_level

        # De we need statistics per start_scanning_uri ?
        # Or global statistics ?
        self.statistics = Statistics()

    def start_scanning_uri(self, base_uri):
        """
        Start scanning a Uri

        @param base_uri: the base uri for the scanning
        @type base_uri: for now, C{str}

        @return: defer fired when the scanning stops
        @rtype C{defer}
        """
        # Creation of data_sources (we shouldn't need that here...)
        crawler = FileSystemCrawler()
        typefinder = MetaTypeFinder([FileTypeFinder()])

        myId = VideoIdentificator()
        myDataSource = TheMovieDBDataSource()
        myId.register_data_source(myDataSource)
        myDataSource = TheTVDBDataSource()
        myId.register_data_source(myDataSource)

        # Creation of the storage and model (we shouldn't need that here...)
        storage = RDF.Storage(storage_name='sqlite', \
                              name='mydb.db', \
                              options_string='')
        final_model = RDF.Model(storage)

        defer_list = []
        all_processing = []
        index = 0

        for file_uri in crawler.crawl(base_uri):
            def after_process_one_file(result, file_uri, model, final_model):
                self.statistics.finished_treating_one(myId.fill_model(model), file_uri)
                for statement in model:
                    final_model.append(statement)
                if self.debug_level == 5:
                    print model
                print ""
                return result

            def store_model(result, final_model):
                final_model.sync()

            def treat_error(failure):
                print "ERROR BASE"
                print failure

            model = RDFModel()
            self.statistics.start_treating_one(file_uri)

            typefinder.get_info_type(file_uri, model=model)
            typefinder.get_info_file(file_uri, model=model)

            dfr = myId.identify_model(model)
            dfr.addCallback(after_process_one_file, file_uri, model, final_model)
            dfr.addErrback(treat_error)
            defer_list.append(dfr)
            all_processing.append(dfr)

            # TODO CREATE defered list
            if len(defer_list) == 5:
                dl = defer.DeferredList(defer_list)
                dl.addCallback(store_model, final_model)
                dl.addErrback(treat_error)
                defer_list = []

        def stop_processing(result, model):
            self.statistics.print_results()
#            print model
            reactor.stop()

        dl = defer.DeferredList(all_processing)
        dl.addCallback(stop_processing, final_model)
        return dl
