# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from core.media_scanner.metadata_manager import MetadataManager, rdf
from core.media_scanner.fs_crawler import FileSystemCrawler

from core.log import Loggable

from twisted.internet import defer, reactor

class ScanJob(object):
    def __init__(self, deferred, base_uri, files_to_scan, **kw):
        super(ScanJob, self).__init__(**kw)
        self._base_uri = base_uri
        self._deferred = deferred
        self._files_to_scan = files_to_scan

class MediaScanner(Loggable):

    class_predicate = rdf['type']

    def __init__(self, metadata_manager, **kw):
        super(MediaScanner, self).__init__(**kw)
        self.manager = metadata_manager
        self._crawler = FileSystemCrawler()
        self.pending_jobs = {}

    def job_finished(self, result, folder):
        self.info("JOB FINISHED %s" % folder)
        del self.pending_jobs[folder]
        return result

    def scan_dir(self, folder):

        files_to_scan = list(self._crawler.crawl(folder))

        list_deferreds = []
        for phile in files_to_scan:
            list_deferreds.append(self.scan_file(phile))

        big_deferred = defer.DeferredList(list_deferreds)
        big_deferred.addCallback(self.job_finished, folder)

        this_job = ScanJob(big_deferred, folder, files_to_scan)

        self.pending_jobs[folder] = this_job

        return big_deferred


    def scan_file(self, uri):
        def bidon(dfr):
            self.warning('scanned %s' % uri)
            dfr.callback(True)
        self.debug('scanning %s' % uri)
        dfr = defer.Deferred()
        reactor.callLater(2, bidon, dfr)
        return dfr

