from twisted.trial.unittest import TestCase

from core.iteration import sequential_iteration, parallel_iteration

class TestException(Exception):
    pass

class IterateTest(TestCase):
    
    def _basic_call(self, iteration):
        source = xrange(10)
        result_list = []
        def save(result):
            result_list.append(result)

        dfr = iteration(source, save)

        def check_results(x):
            self.failUnlessEqual(len(result_list), 10)

        dfr.addCallback(check_results)

        return dfr

    def test_basic_sequential(self):
        return self._basic_call(sequential_iteration)

    def test_basic_parallel(self):
        return self._basic_call(parallel_iteration)

    def test_stop_iteration(self):
        source = xrange(10)
        result_list = []
        def save(result):
            if result >= 4:
                raise StopIteration()
            result_list.append(result)

        dfr = sequential_iteration(source, save)

        def check_results(x):
            self.failUnlessEqual(len(result_list), 4)

        dfr.addCallback(check_results)

        return dfr

    def _test_exception_handling(self, iteration):
        source = xrange(10)
        def raise_exception(result):
            raise TestException()

        dfr = iteration(source, raise_exception)

        def check_results(x):
            # we shouldn't arrive here
            self.fail()

        def errback(failure):
            e=failure.trap(TestException)
            return 42

        dfr.addCallback(check_results)
        dfr.addErrback(errback)

        def check_final(result):
            self.failUnlessEqual(result, 42)

        dfr.addCallback(check_final)

        return dfr

    def _test_sequential_exception_handling(self):
        return self._test_exception_handling(sequential_iteration)

    def test_parallel_exception_handling(self):
        return self._test_exception_handling(parallel_iteration)

