# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from core.media_scanner.data_source import DataSource
from core.media_scanner.rdf_model import RDFModel

from SPARQLWrapper import SPARQLWrapper, JSON

import RDF
import re
import Levenshtein

rdfs = RDF.NS('http://www.w3.org/2000/01/rdf-schema#')

QUERY = """
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX dbonto: <http://dbpedia.org/ontology/>
SELECT ?uri ?label
WHERE {
  ?uri rdfs:label ?label .
  ?uri a dbonto:Film .
  FILTER (regex(?label, ".*%s.*", "i"))
}
"""
# PREFIX dbpprop: <http://dbpedia.org/property/>
# OPTIONAL { ?uri dbpprop:abstract ?synopsis }

class DBPediaDataSource(DataSource):
    name = 'dbpedia'
    mime_type = 'video'
    confidence = 0.9
    type = 'movie'

    def compute_confidence(self, model, search_parameter):
        label = unicode(model.get_target(self.media_id, rdfs['label']))
        label = re.sub(u' \(film\)', u'', label)
        label = label.lower()
        label = re.sub(u'^the ', u'', label)
        return Levenshtein.ratio(search_parameter, label)

    def execute_search(self, search_parameter):
        sparql = SPARQLWrapper("http://dbpedia.org/sparql")

        current_query = QUERY % search_parameter
        sparql.setQuery(current_query)
        sparql.setReturnFormat(JSON)
        print current_query
        results = sparql.query().convert()

        print results
        output = []
        for result in results["results"]["bindings"]:
            possible_model = RDFModel()
            possible_model.mime_type = self.mime_type
            label_statement = RDF.Statement(RDF.Node(self.media_id),\
                                            rdfs['label'],\
                                            RDF.Node(result["label"]["value"]))
            possible_model.append(label_statement)
            uri_statement = RDF.Statement(RDF.Node(self.media_id),\
                                          rdfs['isDescribed'],\
                                          RDF.Node(result["uri"]["value"]))
            possible_model.append(uri_statement)
            synopsis_statement = RDF.Statement(RDF.Node(self.media_id),\
                                               rdfs['synopsis'],\
                                               RDF.Node(result["synopsis"]["value"]))
            possible_model.append(synopsis_statement)
            type_statement = RDF.Statement(RDF.Node(self.media_id),\
                                          rdfs['myType'],\
                                          RDF.Node(self.type))
            possible_model.append(type_statement)
            possible_model.quantity_of_information = 3.0
            output.append(possible_model)
        return output

    def extract_search_parameter(self, model):
        query_statement = RDF.Statement(None, rdfs['prelabel'], None)
        for statement in model.find_statements(query_statement):
            label_node = statement.object
            self.media_id = statement.subject
        return unicode(label_node)

