# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from core.media_scanner.data_source import DataSource
from core.media_scanner.rdf_model import RDFModel

from core.queue import CancellableQueue

import RDF
import Levenshtein
import urllib
import json
import re

from twisted.web.client import getPage

rdfs = RDF.NS('http://www.w3.org/2000/01/rdf-schema#')

def clean_name(name):
    name = name.lower()
    name = re.sub(u' \(film\)', u'', name)
    name = re.sub(u'^the ', u'', name)
    name = name.strip()
    return name


class FreebaseDataSource(DataSource):
    name = 'freebase'
    mime_type = 'video'
    confidence = 2.0
    type = 'movie'

    max_number_of_connexions = 20
    connexion_queue = None
    TIMEOUT = 5.0
    URI_SEARCH = 'http://www.freebase.com/api/service/search?query=%s&type=/film/film'
#    URI_SEARCH = 'http://api.freebase.com/api/service/mqlread?query='\
#                        '{"query":[{"type":"/film/film",'\
#                         '"name~=":"%s",'\
#                         '"id":[],"name":null}]}'

    def __init__(self):
        super(FreebaseDataSource, self).__init__()
        self.connexion_queue = CancellableQueue(self.max_number_of_connexions)

    def execute_search(self, search_parameter, model):
        search_parameter_quoted = urllib.quote(search_parameter)

        # Replace in query string (search_parameter is last)
        search_url_string = self.URI_SEARCH % search_parameter_quoted

        dfr = self._get_page(search_url_string)
        dfr.addCallback(self._treat_search_result, model)

        def treat_error(failure):
            print "ERROR execute_search"
            print failure
            return []

        dfr.addErrback(treat_error)
        return dfr

    def _get_page(self, url):
        def read_file(result):
            return result

        def treat_error(failure):
            print "ERROR _get_page (probable timeout for %s)" % self.name
            return None

        dfr = self.connexion_queue.enqueue(getPage, url, timeout=self.TIMEOUT)
        dfr.addCallback(read_file)
        dfr.addErrback(treat_error)
        return dfr

    def _treat_search_result(self, result, model):
        if result is None:
            return []

        output = []
        json_result = json.loads(result)

        for i in json_result['result']:

            possible_model = RDFModel()
            possible_model.mime_type = self.mime_type
            possible_model.media_id = model.media_id

            uri_id = "http://www.freebase.com/experimental/topic/standard%s" %\
                     i[u'id']

            uri_statement = RDF.Statement(RDF.Node(model.media_id),\
                                          rdfs['isDescribed'],\
                                          RDF.Uri(uri_id))
            possible_model.append(uri_statement)
            label_statement = RDF.Statement(RDF.Uri(uri_id),\
                                          rdfs['label'],\
                                          RDF.Node(i[u'name']))
            possible_model.append(label_statement)
            from_statement = RDF.Statement(RDF.Node(model.media_id),\
                                           rdfs['wasIdentifiedBy'],\
                                           RDF.Node(self.name))
            possible_model.append(from_statement)
#            print possible_model
            output.append(possible_model)
        return output


        return []

    def extract_search_parameter(self, model):
        query_statement = RDF.Statement(None, rdfs['prelabel'], None)
        for statement in model.find_statements(query_statement):
            label_node = statement.object
            model.media_id = statement.subject
#        print unicode(label_node)
        return unicode(label_node)

    def compute_confidence(self, model, search_parameter):
        label = None
        query_statement = RDF.Statement(None, rdfs['label'], None)
        stream = model.find_statements(query_statement)
        for statement in stream:
            label = unicode(statement.object)
        label = clean_name(label)

        search_parameter2 = clean_name(search_parameter)

        ratio_label = Levenshtein.ratio(search_parameter2, label)

        alternate_label = None
        query_statement = RDF.Statement(None, rdfs['alternative_name'], None)
        stream = model.find_statements(query_statement)
        for statement in stream:
            alternate_label = unicode(statement.object)

        if alternate_label is not None:
            alternate_label = clean_name(unicode(alternate_label))
            ratio_alternate = Levenshtein.ratio(search_parameter2, alternate_label)
        else:
            ratio_alternate = 0.0

#        print "%s <> %s : %s/%s [%s]" % (label, search_parameter2, ratio_label, ratio_alternate, alternate_label)
        return max(ratio_alternate, ratio_label)

