# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

import re
import common_regexps

class AtomStopperAlgo(object):

    def _isAtomStopper(self, atom):
        atom_to_keep = True
        # STOP if common rubbish atoms (DVDRiP, etc)
        if atom in common_regexps.pre_clean_atom_stoppers:
            atom_to_keep = False
        # STOP if []{}()
        if common_regexps.atom_stopper_splitters.search(atom):
            atom_to_keep = False
        # STOP if S01E03
        for regex in common_regexps.tvshows_stopper_regexps:
            if regex.search(atom):
                atom_to_keep = False
        # STOP if 2009
        if common_regexps.year_finder.search(atom):
            atom_to_keep = False

        return not atom_to_keep

    def clean_string(self, dirty_string):
        lower_dirty_string = dirty_string.lower()

        cleaned_string = lower_dirty_string
        rubbish_list = common_regexps.loose_rubbish_strings.split('@')
        rubbish_list.sort(key=lambda y: len(y), reverse=True)

#        for regex in rubbish_list:
#            regexp_with_splitters = '(%s|^)%s(%s|$)' % \
#                                     (common_regexps.atom_splitter[1:-1],\
#                                      regex, common_regexps.atom_splitter[1:-1])
#            p = re.compile(regexp_with_splitters)
#            cleaned_string = re.sub(regexp_with_splitters,\
#                                    '\g<1>\g<2>', cleaned_string)

        atoms = re.split(common_regexps.atom_splitter, cleaned_string)

        cleaned_string = ""
        for atom in atoms:
            if self._isAtomStopper(atom):
                break
            elif atom == '':
                pass
            else:
                match = common_regexps.atom_nonstopper_splitters.search(atom)
                if not match:
                    cleaned_string += " %s" % atom
        output = cleaned_string[1:].lower()

        rubbish_list = common_regexps.rubbish_multiword_strings
        rubbish_list.sort(key=lambda y: len(y), reverse=True)
        for regex in rubbish_list:
            output = re.sub(' %s$' % regex, ' ', output)
            output = re.sub(' %s ' % regex, ' ', output)

        return output.strip()


