# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

import re

#########
# COMMON
#########

no_more_digits_after = "([^0-9]+|$)"
no_more_digits_before = "([^0-9]+|^)"

year_finder = re.compile(r'([^0-9]+|^)([1-2][09][0-9]{2})([^0-9]+|$)')

#########################
# ATOM SPLITTER SPECIFIC
#########################

# Atom splitters that are stoppers
atom_stopper_splitters_str = "[\{\[\]\(\)\}]"
atom_stopper_splitters = re.compile(atom_stopper_splitters_str)

# Atom separators that are NOT stoppers
atom_nonstopper_splitters_str = "[-_ \.,~:]"
atom_nonstopper_splitters = re.compile(atom_nonstopper_splitters_str)

# How we split atoms
atom_splitter = '([' + atom_nonstopper_splitters_str[1:-1] + \
                      atom_stopper_splitters_str[1:-1] + '])'

non_meaningful_characters = atom_splitter[1:-1] + "*"

###########
# TV SHOWS
###########

season_matcher = "[s][e]?[a]?[s]?[o]?[n]?"
episode_matcher = "[e][p]?[i]?[s]?[o]?[d]?[e]?"
two_digits = "([0-9]{1,2})"


tvshows_stopper_regexps_str = [
                    # "Sea01"
                    "^" + season_matcher + two_digits + "$",
                    # "Ep12"
                    "^" + episode_matcher + two_digits + "$",
                    # "01x12"
                    "^" + two_digits + "x" +two_digits + "$",
                    # "S01E12"
                    "^" + season_matcher + two_digits + \
                          episode_matcher + two_digits + "$",
                   ]

tvshows_stopper_regexps = [re.compile(regex) for regex in tvshows_stopper_regexps_str]

#print tvshows_stopper_regexps_str

tvshows_matching_regexps_str = [
                      # Lost Season 1 Episode 2.avi
                      # Lost.S01E02.avi
                      # Lost_S01_E02.avi
                      # Lost S01 - E02.avi
                      # Lost.S01.E02.avi
                      # Lost S01xE02.avi
                      season_matcher  + non_meaningful_characters + \
                      two_digits + non_meaningful_characters + \
                      episode_matcher + non_meaningful_characters + \
                      two_digits + no_more_digits_after,

                      # Lost 01x02.avi
                      two_digits + "x"+ two_digits + no_more_digits_after,
                    ]

#print tvshows_matching_regexps_str

tvshows_matching_regexps = [re.compile(regex) for regex in tvshows_matching_regexps_str]

##################
# RUBBISH STRINGS
##################

# Other things that we probably want to remove...

# The following are very strict in the sense that they have a meaning
# for themselves, that is, they could be part of a movie or tv episode title
strict_rubbish_strings = "custom@dutch@internal@limited@pal@proper@retail@"\
            "swedish@german@french@italian@xxx@fragment@release@spanish"

# This is just garbage until someone releases a movie called
# "the revenge of the dvdrip killers"
loose_rubbish_strings = "ac3@dc@divx@dsr@dsrip@dvd@dvdrip@"\
             "dvdscr@dvdscreener@fs@hdtv@multisubs@ntsc@ogg@ogm@"\
             "pdtv@proper@repack@rerip@r3@r5@svcd@nfofix@"\
             "unrated@ws@telesync@telecine@bdrip@720p@1080p@hddvd@"\
             "bluray@xvid@axxo@screener@rock&blueladyrg@"\
             "stealthmaster@nikonxp@repack@phrax@keltz@aac@hls@"\
             "dd@dxo@eng@fra@mp3@fxg@binladen100@tlotr@klaxxon@"\
             "ekolb@rip@saais@wolvish@wal@headcna@zektorm@"\
             "fxm@larceny@moh@greenbud1969@westy1983@bigbro@"\
             "westy1983@imacruel1@2ch@kookoo@h33t@x264"

rubbish_multiword_strings = "dual audio@5 1@(special|unrated) ed(ition)*@the movie@"\
                            "(special )*(extended|collector's) ed(ition)*@"\
                            "torrent y net@cd[1-9]@h.{0,1}264@read nfo@"\
                            "(exclusive )*director('s|s) cut( edition)*@"\
                            "1337x noir@5[ ]{0,1}ch@2[ ]{0,1}ch".split("@")
pre_clean_atom_stoppers = (strict_rubbish_strings + "@" + loose_rubbish_strings).split("@")
pre_clean_atom_stoppers_preregexp = ["^%s$" % string for string in pre_clean_atom_stoppers]
pre_clean_atom_stoppers_regexp = [re.compile(regex) for regex in pre_clean_atom_stoppers_preregexp]

