# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from core.media_scanner.data_source import MetadataSource
from core.uri import get_file_name_from_uri

from plugins.name_parser.name_parser import NameParser
from plugins.ontology.name_spaces import moo

import RDF

from twisted.internet import defer

class NameParserDataSource(MetadataSource):
    name = 'name_parser'
    confidence = 1.0

    def __init__(self, **kw):
        super(NameParserDataSource, self).__init__(**kw)
        self.parser = NameParser()

    def find_statements(self, subject, predicate, object):
        dfr = self._get_file_name(subject)
        dfr.addCallback(self._clean_name, subject)
        return dfr

    def _get_file_name(self, uri_video):
        # Up till now, the only statements with uri_video are
        # [http://www.moovida.com/uuid/1234] [rdf:type] [nmm:Video]
        # [file:///Films/Alien.avi] [moo:dataOf] [http://www.moovida.com/uuid/1234]
        # Note: uri_video = http://www.moovida.com/uuid/1234
        dfr = self._manager.find_statements(None, None, uri_video)

        def find_file_name(statements):
            statements = list(statements)
            uri_file_name = unicode(statements[0].subject.uri)
            return get_file_name_from_uri(uri_file_name)

        dfr.addCallback(find_file_name)
        return dfr

    def _clean_name(self, file_name, uri_video):
        output_model = RDF.Model()

        result = self.parser.get_info_from_name(file_name)

        output_model.append(RDF.Statement(uri_video, \
                                          moo['cleanName'], \
                                          result['clean_name']))
        return output_model

    def execute_search(self, search_parameter, model):
        name_parser = NameParser()

        possible_model = RDFModel()

        result = name_parser.get_info_from_name(unicode(search_parameter))

        cleaned_statement = RDF.Statement(model.media_id,\
                                          rdfs['prelabel'],\
                                          RDF.Node(result['clean_name']))

        possible_model.append(cleaned_statement)

        if result['type'] == 'movie' and result.has_key('possible_date'):
            date_statement = RDF.Statement(model.media_id,\
                                           rdfs['possible_date'],\
                                           RDF.Node(result['possible_date']))
            possible_model.append(date_statement)
        elif result['type'] == 'tvepisode':
            tvshow_statement = RDF.Statement(model.media_id,\
                                           rdfs['might_be'],\
                                           RDF.Node('tvshow'))
            possible_model.append(tvshow_statement)
            season_no_statement = RDF.Statement(model.media_id,\
                                           rdfs['season_no'],\
                                           RDF.Node(result['season_no']))
            possible_model.append(season_no_statement)
            episode_no_statement = RDF.Statement(model.media_id,\
                                           rdfs['episode_no'],\
                                           RDF.Node(result['episode_no']))
            possible_model.append(episode_no_statement)
        return defer.succeed([possible_model])

    def extract_search_parameter(self, model):
        query_statement = RDF.Statement(None, rdfs['file_name'], None)
        for statement in model.find_statements(query_statement):
            label_node = statement.object
            model.media_id = statement.subject
        return unicode(label_node)
