# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from atom_stopper_algo import AtomStopperAlgo
from string_stripper_algo import StringStripperAlgo

import common_regexps
import re

class NameParser(object):
    implementations = [AtomStopperAlgo(),\
                       StringStripperAlgo()]

    def get_info_from_name(self, string):
        output = {}
        file_name_without_ext = string

        for implementation in self.implementations:
            output['clean_name'] = \
                            implementation.clean_string(file_name_without_ext)
            if output['clean_name'] == '' or output['clean_name'] == 'the':
#                print "OTHER!!!"
                pass
            else:
                break

        # TV Shows
        for regexp in common_regexps.tvshows_matching_regexps:
            match = regexp.search(file_name_without_ext.lower())
            if match is None:
                output['type'] = "movie"
            else:
                groups = match.groups()
                if len(groups) >= 2:
                    output['season_no'] = groups[0]
                    output['episode_no'] = groups[1]
                    output['type'] = "tvepisode"
                    break

        # Dates
        if output['type'] == "movie":
            match = common_regexps.year_finder.search(file_name_without_ext)
            if match:
                year = match.groups()
                output['possible_date'] = year[1]
        return output

