# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from plugins.name_parser.name_parser import NameParser
from films import movies
from tvseries import tvepisodes

import sys
import os
import pkg_resources
import Levenshtein

list_of_movies = movies.split('\n')
list_of_tvepisodes = tvepisodes.split('\n')

list_of_references = {'file_name':[], 'label':[]}
f = pkg_resources.resource_stream('plugins.name_parser.tests', 'reference.db.txt')
for line in f:
    splitted_line = line.split("|")
    list_of_references['file_name'].append(splitted_line[0])
    list_of_references['label'].append(splitted_line[1])

def test(args):
    if len(args) == 1:
        list_of_video_names = list_of_movies + list_of_tvepisodes
    elif args[1] == 'series':
        list_of_video_names = list_of_tvepisodes
    elif args[1] == 'movies':
        list_of_video_names = list_of_movies
    elif args[1] == 'reference':
        list_of_video_names = list_of_references['file_name']

    myParser = NameParser()
    i = 0

    successes = []
    partial_successes = []
    failures = []

    for file_name in list_of_video_names:
        result = myParser.get_info_from_name(file_name)
        output = " %s " % file_name
        while len(output) < 80:
           output += " "
        output += "clean [%s] " % result['clean_name']

        if args[1] == 'reference':
            reference_label = list_of_references['label'][i].lower()
            reference_label = reference_label.replace('\n', '')
            output += "ref [%s] " % reference_label
            other_label = result['clean_name']
            ratio = Levenshtein.ratio(reference_label, other_label)
            output += "SCORE=%s " % ratio
            if ratio == 1.0:
                output += "[PERFECT]"
                successes.append(output)
            elif ratio > 0.9:
                output += "[OK]"
                successes.append(output)
            elif ratio > 0.7:
                output += "[PSEEE]"
                partial_successes.append(output)
            else:
                output += "[KO]"
                failures.append(output)
        i += 1

    successes_count = 0.0
    for success in successes:
        print success
        successes_count += 1

    partial_successes_count = 0.0
    for partial_success in partial_successes:
        print partial_success
        partial_successes_count += 1
    failure_count = 0.0
    for failure in failures:
        print failure
        failure_count += 1

    print "%s SUCCESSES" % successes_count
    print "%s PARTIAL SUCCESSES" % partial_successes_count
    print "%s FAILURES" % failure_count
    print "RATIO OF SUCCESS: %s" % ((successes_count + partial_successes_count) / (successes_count + failure_count + partial_successes_count))

if __name__ == '__main__':
    test(sys.argv)
