# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Authors: Fernando Casanova <fcasanova@fluendo.com>

from twisted.trial.unittest import TestCase
from twisted.internet import defer

from plugins.name_parser.metadata_source import NameParserDataSource
from plugins.ontology.name_spaces import moo

import RDF

def stream_wrapper(stream):
    # FIXME: should be put somewhere in utils
    for element in stream:
        yield element

class ManagerMockUp(object):
    def __init__(self):
        super(ManagerMockUp, self).__init__()
        self.cache = RDF.Model()

    def find_statements(self, subject, predicate, object):
        query_statement = RDF.Statement(subject, predicate, object)
        stream = self.cache.find_statements(query_statement)
        return defer.succeed(stream_wrapper(stream))


class MovieDBTest(TestCase):
    test_file_alien = u'file:///home/luser/Videos/ALIEN.[ReDUX].avi'
    test_file_resident = u'file:///home/luser/Videos/Resident%20Evil.avi'

    test_file_alien_blank = RDF.Node()
    test_file_resident_blank = RDF.Node()

    def setUp(self):
        manager = ManagerMockUp()
        self.source = NameParserDataSource(manager=manager)
        manager.cache.add_statement(RDF.Statement(RDF.Uri(self.test_file_alien),
                                                  moo['dataOf'],
                                                  self.test_file_alien_blank))
        manager.cache.add_statement(RDF.Statement(RDF.Uri(self.test_file_resident),
                                                  moo['dataOf'],
                                                  self.test_file_resident_blank))

    def tearDown(self):
        pass

    def test_find_file_name_alien(self):
        dfr = self.source._get_file_name(self.test_file_alien_blank)

        def check_result(file_name):
            self.failUnlessEqual(file_name,\
                                 u'ALIEN.[ReDUX]')
        dfr.addCallback(check_result)
        return dfr

    def test_find_file_name_resident(self):
        dfr = self.source._get_file_name(self.test_file_resident_blank)

        def check_result(file_name):
            self.failUnlessEqual(file_name,\
                                 u'Resident Evil')
        dfr.addCallback(check_result)
        return dfr

    def test_clean_name_simple(self):
        dfr = self.source.find_statements(self.test_file_alien_blank, \
                                          moo['cleanName'], \
                                          None)

        def check_result(statements):
            count = 0

            for statement in statements:
                if statement.predicate == moo['cleanName'] \
                        and statement.subject == \
                            self.test_file_alien_blank:
                    self.failUnlessEqual(statement.object, RDF.Node('alien'))
                    count += 1

            self.failUnlessEqual(count, 1)

        dfr.addCallback(check_result)
        return dfr

