# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Authors: Fernando Casanova <fcasanova@fluendo.com>
#          Guillaume Emont <guillaume@fluendo.com>

from core.media_scanner.xslt_data_source import GetterDataSource, XmlInfoGetter
from core.media_scanner.metadata_manager import Scorer
from core.media_scanner.data_source import add_shortcuts

from plugins.ontology.name_spaces import moo, nie
from plugins.ontology.classes import get_class_statement

import RDF
import re
import Levenshtein

from twisted.internet import task, defer

API_KEY = '90e058d75a94c5b92cd32260a9666b38'

class TheMovieDBScorer(Scorer):
    weight = 2.0
    title = nie['title']
    clean_name = moo['cleanName']

    @defer.inlineCallbacks
    def get_score(self, model, original_subject):
        # we score by levenshtein-comparing the label held in statements and
        # a clean name we ask to the manager

        # Find the labels to compare: title
        query_statement = RDF.Statement(original_subject, self.title, None)
        stream = model.find_statements(query_statement)
        for statement in stream:
            title_node = unicode(statement.object).lower()

        # Find the labels to compare: clean_name
        statements = yield self.manager.find_statements(original_subject,\
                                                        self.clean_name,\
                                                        None)
        statements = list(statements)
        clean_name_node = unicode(statements[0].object).lower()

        # Compute ratio
        ratio = Levenshtein.ratio(clean_name_node, title_node)
        defer.returnValue(ratio)


class TheMovieDBDataSource(GetterDataSource):
    name = 'themoviedb'
    handled_classes = [moo['Movie']]

    path_to_current = 'plugins.themoviedb.xslt'

    movie_search_xslt = "%s:%s" % (path_to_current, 'Movie.search.xsl')
    movie_getinfo_xslt = "%s:%s" % (path_to_current, 'Movie.getInfo.xsl')

    MOVIE_DETAILS_URI = 'http://api.themoviedb.org/2.1/Movie.getInfo/en/xml/'
    URI_SEARCH = ('http://api.themoviedb.org/2.1/Movie.search/en/xml/%s/' % API_KEY) + '%s'
    stylesheet_args = {'API_KEY': '"%s"' % API_KEY,
                       'BASE_URI': '"%s"' % MOVIE_DETAILS_URI,
                       }

    # FIXME: do we really need separate stylesheets for getInfo and search?
    xslt_dict = {
        '.*Movie.search.*': (movie_search_xslt, stylesheet_args),
        '.*Movie.getInfo.*': (movie_getinfo_xslt, stylesheet_args),
    }

    def __init__(self, **kw):
        getter = XmlInfoGetter(xslt_dict=self.xslt_dict)
        super(TheMovieDBDataSource, self).__init__(getter=getter, **kw)

    def _query_to_uri(self, subject, predicate, object):
        # FIXME: implement for real, in a non-blocking way
        dfr = self._manager.get_class(subject)

        # FIXME: the code below looks bad
        def construct_uri(klass):
            if klass == moo['Movie'] \
                        and predicate == nie['title'] \
                        and object is None:
                dfr = self._manager.find_statements(subject,
                                                    moo['cleanName'],
                                                    None)
                def make_search_uri_from_label(label_statements):
                    label = label_statements.next().object
                    return self.URI_SEARCH % label

                dfr.addCallback(make_search_uri_from_label)
                return dfr

            # TODO: handle the rest of the stuff

        dfr.addCallback(construct_uri)

        return dfr

    def _select_result(self, models, subject, predicate, object):
        scorer = TheMovieDBScorer(self._manager)
        highest_score = 0.0
        result = None

        # list of (score, model)
        scores = []

        def iterate():
            for model in models:
                statement = get_class_statement(model)
                model = add_shortcuts(model, statement.subject, subject)
                dfr = scorer.get_score(model, subject)
                def add_score(score):
                    if score is not None:
                        scores.append((score, model))
                dfr.addCallback(add_score)
                yield dfr

        dfr = task.coiterate(iterate())

        def get_result(x):
            # suboptimal to sort the whole list to get the biggest
            sorted_scores = sorted(scores, key=lambda x: x[0], reverse=True)
            best_score, best_model = sorted_scores[0]
            return best_model

        dfr.addCallback(get_result)

        return dfr
