# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Guillaume Emont <guillaume@fluendo.com>

from plugins.themoviedb.metadata_source import TheMovieDBDataSource

from twisted.trial.unittest import TestCase
from twisted.internet import defer

import RDF

from plugins.ontology.name_spaces import nie, moo

clean_name_predicate = moo['cleanName']

def stream_wrapper(stream):
    # FIXME: should be put somewhere in utils
    for element in stream:
        yield element

class ManagerMockUp(object):
    def __init__(self):
        super(ManagerMockUp, self).__init__()
        self.cache = RDF.Model()

    def get_class(self, node):
        return defer.succeed(moo['Movie'])

    def find_statements(self, subject, predicate, object):
        query_statement = RDF.Statement(subject, predicate, object)
        stream = self.cache.find_statements(query_statement)
        return defer.succeed(stream_wrapper(stream))


class MovieDBTest(TestCase):
    test_file_alien = u'file:///home/luser/Vidéos/alien.avi'
    test_file_resident = u'file:///home/luser/Vidéos/resident%20evil.avi'
    def setUp(self):
        manager = ManagerMockUp()
        self.source = TheMovieDBDataSource(manager=manager)
        manager.cache.add_statement(RDF.Statement(RDF.Uri(self.test_file_alien),
                                                  clean_name_predicate,
                                                  'alien'))
        manager.cache.add_statement(RDF.Statement(RDF.Uri(self.test_file_resident),
                                                  clean_name_predicate,
                                                  'resident evil'))

    def tearDown(self):
        pass

    def test_query_to_uri(self):
        dfr = self.source._query_to_uri(RDF.Uri(self.test_file_alien),
                                        nie['title'],
                                        None)
        def check_result(uri):
            self.failUnlessEqual(uri, TheMovieDBDataSource.URI_SEARCH % 'alien')

        dfr.addCallback(check_result)

        # FIXME: add more queries

        return dfr

    def test_select_result(self):
        # TODO
        pass

    def test_find_statements_alien(self):
        # FIXME: mark as a functional test
        dfr = self.source.find_statements(RDF.Uri(self.test_file_alien),
                                          nie['title'],
                                          None)

        def check_result(statements):
            count = 0

            for statement in statements:
                if statement.predicate == nie['title'] \
                        and statement.subject == \
                            RDF.Node(RDF.Uri(self.test_file_alien)):
                    self.failUnlessEqual(statement.object, RDF.Node('Alien'))
                    count += 1

            self.failUnlessEqual(count, 1)
        dfr.addCallback(check_result)

        return dfr

    def test_find_statements_resident(self):
        # FIXME: mark as a functional test
        dfr = self.source.find_statements(RDF.Uri(self.test_file_resident),
                                          nie['title'],
                                          None)

        def check_result(statements):
            count = 0

            for statement in statements:
                if statement.predicate == nie['title'] \
                        and statement.subject == \
                            RDF.Node(RDF.Uri(self.test_file_resident)):
                    self.failUnlessEqual(statement.object, RDF.Node('Resident Evil'))
                    count += 1

            self.failUnlessEqual(count, 1)
        dfr.addCallback(check_result)

        return dfr
