# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Fernando Casanova <fcasanova@fluendo.com>

from core.media_scanner.xslt_data_source import XsltDataSource

import RDF
import re
import Levenshtein

from twisted.internet import defer

rdfs = RDF.NS('http://www.w3.org/2000/01/rdf-schema#')

API_KEY = 'F3BC89000AEB8902'

def clean_name(name):
    name = name.lower()
    name = re.sub(u'^the ', u'', name)
    name = name.strip()
    return name

class TheTVDBDataSource(XsltDataSource):
    name = 'thetvdb'
    mime_type = 'video'
    confidence = 1.0
    type = 'series'

    path_to_current = 'plugins.thetvdb.xslt'
    identificator_transform = (path_to_current, 'identificator.xsl')
    filler_transform = (path_to_current, 'filler.xsl')

    TVSERIES_DETAILS_URI = 'http://www.thetvdb.com/api/%s/series' % API_KEY
    stylesheet_args =  { 'BASE_URI': ('"%s"' % TVSERIES_DETAILS_URI) }

    URI_SEARCH = 'http://www.thetvdb.com/api/GetSeries.php?seriesname=%s'

    max_number_of_connexions = 10
    TIMEOUT = 5

    def compute_confidence(self, model, search_parameter):
        label = None
        query_statement = RDF.Statement(None, rdfs['label'], None)
        stream = model.find_statements(query_statement)
        for statement in stream:
            label = unicode(statement.object)
        label = clean_name(label)

        search_parameter2 = clean_name(search_parameter)

        ratio_label = Levenshtein.ratio(search_parameter2, label)

        alternate_label = None
        query_statement = RDF.Statement(None, rdfs['alternative_name'], None)
        stream = model.find_statements(query_statement)
        for statement in stream:
            alternate_label = unicode(statement.object)

        if alternate_label is not None:
            alternate_label = clean_name(unicode(alternate_label))
            ratio_alternate = Levenshtein.ratio(search_parameter2, alternate_label)
        else:
            ratio_alternate = 0.0

#        print "%s <> %s : %s/%s [%s]" % (label, search_parameter2, ratio_label, ratio_alternate, alternate_label)
        return max(ratio_alternate, ratio_label)
