import RDF
import mimetypes, urlparse, urllib, subprocess

import os

dc = RDF.NS('http://purl.org/dc/elements/1.1/')
rdfs = RDF.NS('http://www.w3.org/2000/01/rdf-schema#')

class TypeFinder(object):
    mime_type_predicate = dc['format']

    def get_info_type(self, uri, model):
        """
        DOCME

        @return: a model with all the statements we could find about uri
        @rtype: RDF.Model
        """
        mime = self._get_mime(uri)
        if mime is not None:
            statement = RDF.Statement(RDF.Uri(uri),
                                      self.mime_type_predicate,
                                      RDF.Node(mime))
            model.append(statement)
            type = mime.split('/')
            if type[0] == 'video':
                statement = RDF.Statement(RDF.Uri(uri),
                                          rdfs['type'],
                                          RDF.Node(type[0]))
                model.append(statement)
        return model

    def get_info_file(self, uri, model):
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(uri)
        path = urllib.unquote(path)
        base_name = os.path.basename(path)
        name_file, ext = os.path.splitext(base_name)
        statement_prelabel = RDF.Statement(RDF.Uri(uri),
                                           rdfs['file_name'],
                                           RDF.Node(name_file))
        statement_extension = RDF.Statement(RDF.Uri(uri),
                                            dc['hasExtension'],
                                            RDF.Node(ext[1:]))
        model.append(statement_prelabel)
        model.append(statement_extension)

    def _get_mime(self, uri):
        pass


class ExtensionTypeFinder(TypeFinder):
    """
    Typefinder using the mimetypes module, getting information from extension.
    """

    def _get_mime(self, uri):
        mime_type, encoding = mimetypes.guess_type(uri)
        return mime_type

class FileTypeFinder(TypeFinder):
    def _get_mime(self, uri):
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(uri)
        path = urllib.unquote(path)
        command = ['file', '-bi', path]

        pipe = subprocess.Popen(command, stdout=subprocess.PIPE).stdout
        result = pipe.read()
        pipe.close()

        return result.strip()

class MetaTypeFinder(TypeFinder):
    def __init__(self, typefinders):
        super(MetaTypeFinder, self).__init__()
        self._typefinders = typefinders

    def get_info_type(self, uri, model):
        for typefinder in self._typefinders:
            typefinder.get_info_type(uri, model)
        return model

if __name__ == '__main__':
    import sys
    from crawler.fs_crawler import FileSystemCrawler

    base_uri = sys.argv[1]
    crawler = FileSystemCrawler()
    typefinder = MetaTypeFinder([FileTypeFinder()])

    from id.identificator import Identificator
    from id.dbpedia_data_source import DBPediaDataSource

    myId = Identificator('video')
    myDataSource = DBPediaDataSource()
    myId.register_data_source(myDataSource)

    for file_uri in crawler.crawl(base_uri):
        storage = RDF.MemoryStorage()
        model = RDF.Model(storage=storage)
        typefinder.get_info_type(file_uri, model=model)
        typefinder.get_info_file(file_uri, model=model)
        myId.identify_model(model)

    for statement in model.as_stream():
        print statement

