from core.metadata_manager import MetadataManager, \
         SourceScorer, LevenshteinScorer

import RDF

# the "nature" predicate tells us whether it's a film, a tv show, a music
# track, etc...
nature_predicate = our_ontology['nature']

needed_metadata = {
# klass -> (known subject predicates, known object predicates)
    our_ontology['file']: ([our_ontology['describedBy']], [])
    our_ontology['media']: ([our_ontology['describedBy']], [])
    our_ontology['tvshow']: ([our_ontology['label'], our_ontology['series'],
                              our_ontology['season']],
                             [our_ontology['actsIn']]),
    our_ontology['film']:   ([our_ontology['label'],
                              our_ontology['releaseYear']],
                             [our_ontology['actsIn']]),
    our_ontology['music_track']: ([our_ontology['author'],
                                   our_ontology['interpret'],
                                   our_ontology['label']],
                                  [our_ontology['contains']]),
}

# example result for a file called alien.avi with the loosely defined
# and probably broken ontology suggested above
#
# /home/guijemont/Videos/alien.avi a file
# /home/guijemont/Videos/alien.avi describedBy media0
# media0 a film
# media0 describedBy http://api.themoviedb.org/2.1/Movie.getInfo/en/xml/01234/348
# http://.../348 label 'Alien'
# http://.../348 releaseYear 1979
# Actor0 a actor
# Actor0 describedBy http://www.themoviedb.org/person/10205
# http://www.themoviedb.org/person/10205 label 'Sigourney Weaver'
# Actor0 actsIn http://.../348



# various notes:
# our_ontology['contains'] is quite generic, and applies to
# album -(contains)-> track
# it's up to the data source to check whether it can reply to a "contains"
# request, e.g. by checking the nature of the object of the request (which in
# our case will be cached)
#
# a source should not return a value that is already in the cache prior to it
# being called, but it can return an alternative value
#
# Issue with that scanning algorithm: the needed_metadata information should
# be carefully constructed so as to avoid downloading the whole of internet
# while scanning

class_statement = our_ontology['isA']

def get_class(manager, node):
    def request_class(node):
        statements = manager.find_statements(node, [class_statement], None,
                                             use_cache=False)
        # we assume there is only one class_statement arc from a given node
        if statements:
            class_node = statements.next()
        else:
            class_node = None

        return class_node

    class_node = None

    while True:
        new_class_node = request_class(node)
        if new_class_node is None or new_class_node == class_node:
            return class_node
        
def get_class_hierarchy(manager, class_node):
    # return a list of class nodes (uris?) from which class_node inherits
    pass

def new_nodes(statements, known_nodes):
    result = []
    for statement in statements:
        for node in (statement.subject, statement.object):
            if node not in known_nodes:
                result.append(node)
    return result

# proof of concept "algorithm": that's not real code (yet)
def scan_file(uri):
    manager = MetadataManager()
    manager.add_scorer(SourceScorer())
    manager.add_scorer(LevenshteinScorer())
    # add sources here

    file_node = RDF.Node(RDF.Uri(uri))
    manager.add_statements(RDF.Statement(file_node, class_statement,
                                         our_ontology['file']))

    known_nodes = [file_node]

    def scan_node(node)
        # do scanning actions on node, return the new nodes found in the
        # process
        class_node = get_class(manager, node)
        klass = str(class_node.uri)
        known_subject_predicates, known_object_predicates = \
                                                        needed_metadata[klass]

        statements = []
        result =  manager.find_statements(file_node, known_subject_predicates,
                                          None)
        statements.extend(result)
        result = manager.find_statements(None, known_object_predicates,
                                         file_node)
        statements.extend(result)

        discovered = new_nodes(statements, known_nodes):
        known_nodes.extend(discovered)
        for node in discovered:
            scan_node(node)

    scan_node(file_node)
